<?php
//Helper functions
/**
 * Get top 100 used passwords
 * @return string[]
 */
function getTopPassword()
{
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, "https://www.passwordrandom.com/most-popular-passwords");
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_USERAGENT, "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.13 (KHTML, like Gecko) Chrome/0.A.B.C Safari/525.13");
    $data = curl_exec($ch);
    curl_close($ch);

    $dom = new domDocument;

    @$dom->loadHTML($data);
    $dom->preserveWhiteSpace = false;
    $table = $dom->getElementById('cntContent_lstMain');
    $rows = $table->getElementsByTagName('tr');

    $passwords = [];
    foreach ($rows as $row) {
        /** @var DOMElement $row */
        $cols = $row->getElementsByTagName('td');
        $element = $cols->item(1);

        if ($element !== null) {
            $passwords[] = $element->nodeValue;
        }
    }
    return $passwords;
}

/**
 * @param $pass string password you would like to know count in the api.pwnedpasswords.com database
 * @return int|null null if there was not found record for this password
 */
function getPasswordCount($pass) {
    $sha = strtoupper(sha1($pass));
    $prefix = substr($sha, 0, 5);
    $suffix = substr($sha, 5);

    $data = file_get_contents("https://api.pwnedpasswords.com/range/" . $prefix);
    $records = explode("\r\n", strtoupper($data));
    foreach ($records as $record) {
        $subRecords = explode(':', $record);
        if (count($subRecords) == 2) {
            if ($subRecords[0] == $suffix) {
                return intval($subRecords[1]);
            }
        }
    }
    return null;
}


//Application code
//If you pass some arguments, process only them
if ($argc > 1) {
    for ($i = 1; $i < $argc; $i++) {
        $password = $argv[$i];
        $c = getPasswordCount($password);
        echo $password . " -> " .($c ?: 'Not found') . PHP_EOL;
    }
    exit();
}

//Obtain top passwords
$topPassword = getTopPassword(); //$topPassword = ['P@ssw0rd'];

//Echo table in CSV style
echo 'Password,Count' . PHP_EOL;
foreach ($topPassword as $password) {
    $c = getPasswordCount($password);
    echo $password . "," .($c ?: -1) . PHP_EOL;
}
